# -------------------------------------------------------------------------
# Copyright (c) 2022-present, SeetaCloud, Co.,Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------
"""Deprecation utility."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import re

from codewithgpu.utils import decorator
from codewithgpu.utils import logging

# Allow deprecation warnings to be silenced temporarily with a context manager.
_PRINT_DEPRECATION_WARNINGS = True

# Remember which deprecation warnings have been printed already.
_PRINTED_WARNING = {}


def _validate_callable(func, decorator_name):
    if not hasattr(func, '__call__'):
        raise ValueError(
            '%s is not a function. If this is a property, make sure'
            ' @property appears before @%s in your source code:'
            '\n\n@property\n@%s\ndef method(...)' % (
                func, decorator_name, decorator_name))


def _validate_deprecation_args(date, instructions):
    if date is not None and not re.match(r'20\d\d-[01]\d-[0123]\d', date):
        raise ValueError('Date must be YYYY-MM-DD.')
    if not instructions:
        raise ValueError('Don\'t deprecate things without conversion instructions!')


def _get_qualified_name(function):
    # Python 3
    if hasattr(function, '__qualname__'):
        return function.__qualname__
    # Python 2
    if hasattr(function, 'im_class'):
        return function.im_class.__name__ + '.' + function.__name__
    return function.__name__


def deprecated(date, instructions, warn_once=True):
    _validate_deprecation_args(date, instructions)

    def decorated(inner_func):
        _validate_callable(inner_func, 'deprecated')

        def wrapper(*args, **kwargs):
            if _PRINT_DEPRECATION_WARNINGS:
                if inner_func not in _PRINTED_WARNING:
                    if warn_once:
                        _PRINTED_WARNING[inner_func] = True
                    logging.warning(
                        '{} (from {}) is deprecated and will be removed {}.\n'
                        'Instructions for updating:\n{}'.format(
                            _get_qualified_name(inner_func),
                            inner_func.__module__,
                            'in a future version' if date is None else ('after %s' % date),
                            instructions))
                return inner_func(*args, **kwargs)

        return decorator.make_decorator(inner_func, wrapper)

    return decorated


def not_installed(package=''):
    """Return a dummy function for the package that is not installed."""
    def dummy_fn(*args, **kwargs):
        raise ImportError('Package <%s> is required but not installed.' % package)
    return dummy_fn


class NotInstalled(object):
    """Return a dummy object for the package that is not installed."""

    def __init__(self, package=''):
        self._package = package

    def __getattr__(self, item):
        raise ImportError('Package <%s> is required but not installed.' % self._package)
